/*
 * This file contains the functions that are used by
 * the GUI callback stubs.
 */

#include<Xm/Xm.h>
#include<unistd.h>
#include<signal.h>
#include<stdio.h>
#include "dspDemo_defines.h"
#include "dspDemo_externs.h"
#include "dspDemo_globals.h"
#define DEBUG
/*
 * FUNCTION PROTOTYPES
 */
int   getIntFromTextField( Widget textField);
int   calculateNewDataSize( int dataSize, int direction);
int   calculateNewNumberOfProcesses( int currentNumberOfPrcoesses, int direction );
void  putIntToTextField( Widget a , int b);
void  putFloatToTextField( Widget a_textField , float a_float);
void  updateStatChangeTextField( Widget textField_changeInStat, float percentChange );
int   launchAnalysis(  int algorithm, int dataSize, int analysisType );
void  setButtonsState( int running );
int   isAnalysisRunning( );
char  *convertAlgorithmToString( int algorithm );
char  *getSimFileName( int algorithm, int dataSize, int local_numberOfProcesses );
char  *getStatFileName( int algorithm, int dataType );
int   readAlgorithmSetting( int *algorithm, int *dataSize, int *local_numberOfProcesses,
                            int *dataType, int *algorithmForm );
int   isAnalysisRunning( );
int   updateStatistics( );
float calculateChange( int base, int new );
char  *convertDataTypeToString( int dataType );
int   findPowerOfTwo( int a_number );
int   whatPowerOfTwo( int a_number );
int   updateChangeInStats( );



/*
 *       getIntFromTextField
 *
 * This function reads the string from the specfied textField
 *  Widget.  It then converts the string to an integer value
 *  and returns this value.
 *
 *  Input  : textField - the textField containing an integer represented
 *                      as a string
 *  Output : the integer value the string represented in the textField
 */

int   getIntFromTextField( Widget textField )
{
    char *textField_string;
    int   integer_value;

  /*
   * read contents of textField and convert to int
   */
    textField_string = (char *)XmTextFieldGetString( textField );
    integer_value = atoi( textField_string );

  /*
   * free memory alloc'ed by Xm function
   */
    XtFree( textField_string );

    return( integer_value );
}



/*
 *     calculateNewDataSize
 *
 * This function caluculates a new dataSize based on the current
 * dataSize and the INCREMENT/DECREMENT option specifed.  The new
 * data size is the nearest power of 2 in the specified direction.
 * The function imposes limits of MAX_DATASIZE and MIN_DATASIZE.
 *
 *  Input  : currentDataSize - the current date length
 *           direction - whether to INCREMENT or DECREMENT the dataSize
 *  Output : the newDataSize
 */

int
calculateNewDataSize( int currentDataSize , int direction )
{
     int newDataSize;
     int powerOfTwo;


     powerOfTwo = findPowerOfTwo( currentDataSize );
     /*
      * increase/decrease value to the nearest power of two  &
      * update the data size text field
      */
      if( direction == INCREMENT )
        powerOfTwo *= 2;
      else
        powerOfTwo /= 2;

     /*
      * Only change newDataSize if new value is < MAX_DATASIZE
      * and > MIN_DATASIZE
      */
      if(( powerOfTwo <= MAX_DATASIZE )&&( powerOfTwo >= MIN_DATASIZE ))
        newDataSize = (int)powerOfTwo;
      else
        newDataSize = currentDataSize;

      return( newDataSize );
}



/*
 *     calculateNewNumberOfProcesses
 *
 * This function caluculates a new dataSize based on the current
 * dataSize and the INCREMENT/DECREMENT option specifed.  The new
 * data size is the nearest power of 2 in the specified direction.
 * The function imposes limits of MAX_DATASIZE and MIN_DATASIZE.
 *
 *  Input  : currentNumberOfProcesses - the current date length
 *           direction - whether to INCREMENT or DECREMENT the dataSize
 *  Output : the newNumberOfProcesses
 */

int
calculateNewNumberOfProcesses( int currentNumberOfProcesses , int direction )
{
     int newNumberOfProcesses;
     int powerOfTwo;

     powerOfTwo = findPowerOfTwo( currentNumberOfProcesses );

     /*
      * increase/decrease value to the nearest power of two  &
      * update the data size text field
      */
      if( direction == INCREMENT )
        powerOfTwo *= 2;
      else
        powerOfTwo /= 2;

     /*
      * Only change newNumberOfProcesses if new value is < MAX_NUMBEROFPROCESSES
      * and > MIN_NUMBEROFPROCESSORS
      */
      if(( powerOfTwo <= MAX_NUMBEROFPROCESSES )&&( powerOfTwo >= MIN_NUMBEROFPROCESSES ))
        newNumberOfProcesses = (int)powerOfTwo;
      else
        newNumberOfProcesses = currentNumberOfProcesses;

      return( newNumberOfProcesses );
}



/*
 *      putIntToTextField
 *
 * This function writes an integer to a textField. It first takes
 * converts the integer to a string and then writes the string to
 * the specfied textField Widget.
 *
 * Input  : textField - the textField to which the integer to be written
 *          integer_value - the integer to be displayed in the textField
 */
void
putIntToTextField( Widget textField, int integer_value )
{
     char textField_string[MAX_TEXTFIELD_STRINGLEN];

     sprintf( textField_string, "%i", integer_value );
     XmTextFieldSetString( textField, textField_string );

}


/*
 *      putFloatToTextField
 *
 * This function writes an float to a textField. It first takes
 * converts the float to a string and then writes the string to
 * the specfied textField Widget.
 *
 * Input  : textField - the textField to which the float to be written
 *          float_value - the float to be displayed in the textField
 */
void
putFloatToTextField( Widget textField, float float_value )
{
     char textField_string[MAX_TEXTFIELD_STRINGLEN];

     sprintf( textField_string, "%12.1f", float_value );
     XmTextFieldSetString( textField, textField_string );

}
 

/*
 *  launchAnalysis
 *
 *  This routine initiates an analysis simulation.  It allows
 *  for a FREE_RUNning analaysis where the events occur continuously
 *  and are driven by the alarm timer.  It also allows for a STEP_THRU
 *  analysis where the events are driven by the user action of
 *  pressing the Step button.
 */
int
launchAnalysis(  int algorithm, int dataSize, int analysisType )
{
  char * simFileName;
  
  XClearWindow( display, win );
  InitGraph( );
  simFileName = getSimFileName( algorithm, dataSize, numberOfProcesses );
/*   printf("simfile = %s\n\n", simFileName ); */

  if( readLogFile( simFileName ) == ERROR )
    return( ERROR );

  /* calculate the percentage of total time each process takes */
  calculatePercentTime( event_list );
/*  printf(" time 1 is %f \n",event_list->time ); */

  /* if RUN button was pressed start timer for a FREE_RUN */
  if( analysisType == FREE_RUN )
    setEventTimer( SCALE_FACTOR );

  /* free memory allocated by getSimFileName */
  free( simFileName );

  return( NO_ERROR );
}


/*
 *
 * updateStatistics
 *
 */
int
updateStatistics( )
{
  int algorithm;
  int dataSize;
  int local_numberOfProcesses;
  int dataType;
  int algorithmForm;
  int algorithmName;
  char *statFileName;
  STATS_TYPE newStats;
  
  readAlgorithmSetting( &algorithm, &dataSize, &local_numberOfProcesses,
                         &algorithmForm, &dataType );
  statFileName = getStatFileName( algorithm, dataType );
/*  printf(" type filename = %s\n",statFileName ); */
  readStatsFromFile( statFileName, &newStats, dataSize );
  writeStatsToGUI( newStats );

  updateChangeInStats( );

}

/*
 * updateBaselineStats( )
 *
 */
int
updateChangeInStats( )
{
  STATS_TYPE newStats;
  float temp;

  readStatsFromGUI( &newStats );
/*   printf(" %d \n",newStats.float_adds); */
  
  /* calculate change in float adds */

  change_in_stats.float_adds = calculateChange( baseline_stats.float_adds, newStats.float_adds );
/*                               baseline_stats.float_adds; */
                              

  /* calculate change in float mults */
  change_in_stats.float_mults = calculateChange( baseline_stats.float_mults,
                                                 newStats.float_mults );
  /* calculate change in int adds */
  change_in_stats.int_adds = calculateChange( baseline_stats.int_adds,
                                              newStats.int_adds );
  /* calculate change in int mults */
  change_in_stats.int_mults = calculateChange( baseline_stats.int_mults,
                                               newStats.int_mults );
  /* calculate change in memory used */
  change_in_stats.memory_used = calculateChange( baseline_stats.memory_used,
                                                 newStats.memory_used );
  /* calculate change in shifts */
  change_in_stats.number_of_shifts = calculateChange( baseline_stats.number_of_shifts,
                                                      newStats.number_of_shifts );
/*  printf("write change to GUI\n"); */
  /* write change in stats to textFields */
  updateStatChangeTextField( textField_changeIn_memoryUsed,
                             change_in_stats.memory_used );
  updateStatChangeTextField( textField_changeIn_shifts,
                             change_in_stats.number_of_shifts );
  updateStatChangeTextField( textField_changeIn_floatingPoint_adds,
                             change_in_stats.float_adds);
  updateStatChangeTextField( textField_changeIn_floatingPoint_mults,
                             change_in_stats.float_mults );
  updateStatChangeTextField( textField_changeIn_integer_adds,
                             change_in_stats.int_adds );
  updateStatChangeTextField( textField_changeIn_integer_mults,
                             change_in_stats.int_mults );

}

/*
 * updateStatChangeTextField
 *
 */
void
updateStatChangeTextField( Widget textField_changeInStat, float percentChange )
{
  if( percentChange == INF_FLAG )
    XmTextFieldSetString( textField_changeInStat, INFINITY_STRING );
  else
    putIntToTextField( textField_changeInStat, (int)percentChange );
}


/*
 * calculateChange
 *
 * This routine calculates the PERCENT change from base to new
 */
float
calculateChange( int base, int new )
{
  float change;
  if( base != 0)
    change = (( new - base )*100) / (float)base;
  else {
    if( new == 0 )
      change = 0;
    else
      change = INF_FLAG;
  }
  return( change );
}


/*
 * setNewBaseline
 *
 */
int
setNewBaseline( )
{
  STATS_TYPE newStats;
  char  *algorithmName;
  int order;
  char baselineSetting[64];
  Widget algorithm_buttonChosen;

/*  newStats = (STATS_TYPE *)malloc( sizeof( STATS_TYPE )); */
  
  /* read stats from GUI */
  readStatsFromGUI( &newStats );

  /* assign current stats to baseline */
  baseline_stats.float_adds = newStats.float_adds;
  baseline_stats.float_mults = newStats.float_mults;
  baseline_stats.int_adds = newStats.int_adds;
  baseline_stats.int_mults = newStats.int_mults;
  baseline_stats.number_of_shifts = newStats.number_of_shifts;
  baseline_stats.memory_used = newStats.memory_used;
/*  printf(" baseline fp add = %d fp mult = %d intadd = %d int mults = %d #shift = %d memused = %d\n",
          baseline_stats.float_adds,  baseline_stats.float_mults,  baseline_stats.int_adds,  baseline_stats.int_mults,
         baseline_stats.number_of_shifts,  baseline_stats.memory_used );
*/
    /* Read the selected algorithm from pulldown menu */

  XtVaGetValues( pulldown_algorithm,
                 XmNmenuHistory,&algorithm_buttonChosen,
                 NULL);


  /* determine algorithm chosen */

  if( algorithm_buttonChosen == pushButton_algorithm_dft )
    algorithmName = convertAlgorithmToString( DFT );
  if( algorithm_buttonChosen == pushButton_algorithm_radix2 )
    algorithmName = convertAlgorithmToString( RADIX2 );
  if( algorithm_buttonChosen == pushButton_algorithm_radix4 )
    algorithmName = convertAlgorithmToString( RADIX4 );
  if( algorithm_buttonChosen == pushButton_algorithm_splitradix )
    algorithmName = convertAlgorithmToString( SPLITRADIX );
  if( algorithm_buttonChosen == pushButton_algorithm_qft )
    algorithmName = convertAlgorithmToString( QFT );
  if( algorithm_buttonChosen == pushButton_algorithm_ditf )
    algorithmName = convertAlgorithmToString( DITF );
  if( algorithm_buttonChosen == pushButton_algorithm_fht )
    algorithmName = convertAlgorithmToString( FHT );

  
  order = getIntFromTextField( textField_dataSize );

  sprintf( baselineSetting, "%i point %s",order,algorithmName );
  XmTextFieldSetString( textField_baselineSetting, baselineSetting );
  XtFree( algorithmName );

  updateChangeInStats( );
}

/*
 * readStatstFromGUI
 */
int
readStatsFromGUI( STATS_TYPE *newStats )
{
  newStats->float_adds = getIntFromTextField( textField_floatingPointAdditions );
  newStats->float_mults = getIntFromTextField( textField_floatingPointMultiplies );
  newStats->int_adds = getIntFromTextField( textField_integerAdditions );
  newStats->int_mults = getIntFromTextField( textField_integerMultiplies );
  newStats->number_of_shifts = getIntFromTextField( textField_numberOfShifts );
  newStats->memory_used = getIntFromTextField( textField_memoryUsed );
}


/*
 * writeStatsToGUI
 *
 */
int
writeStatsToGUI( STATS_TYPE newStats )
{
  
/*  printf(" float mult %d adds %d  | int mult %d  adds %d | \n",newStats.float_mults,
         newStats.float_adds,newStats.int_mults,newStats.int_adds); */
  putIntToTextField( textField_floatingPointMultiplies, newStats.float_mults );
  putIntToTextField( textField_floatingPointAdditions, newStats.float_adds );
  putIntToTextField( textField_integerMultiplies, newStats.int_mults );
  putIntToTextField( textField_integerAdditions, newStats.int_adds );
  putIntToTextField( textField_numberOfShifts, newStats.number_of_shifts );
  putIntToTextField( textField_memoryUsed, newStats.memory_used );
}

/*
 * getSimFileName
 *
 * This function generates the name of the simulation file
 * based on the algorithm, data size, and number of processes.
 * The memory in which the simulation filename is stored is allocaed
 * by this routine and should be free by the callling routine after use.
 *
 */
char *
getSimFileName( int algorithm, int dataSize, int local_numberOfProcesses )
{
  char *fileName;
  char *algorithmName;

  /* allocate the memory for the filename */
  fileName = (char *)malloc( MAX_FILENAME_LEN * sizeof( char ) );

  /* convert algorithm to string */
  algorithmName = convertAlgorithmToString( algorithm );

  /* build path/filename for corresponding parameters */
  sprintf( fileName, "%s%s_%d_%d.dat", SIM_FILE_HOME,
           algorithmName, local_numberOfProcesses, dataSize );

  /* remember to free memory allocated by convertAlgo.. */
  free( algorithmName );
  
  return( fileName );
  
}



/*
 * getStatFileName
 *
 * This function generates the name of the statistics file
 * based on the algorithm and data type
 * The memory in which the simulation filename is stored is allocaed
 * by this routine and should be free by the callling routine after use.
 *
 */
char *
getStatFileName( int algorithm, int dataType )
{
  char *fileName;
  char *algorithmName;
  char *dataTypeName;

  /* allocate the memory for the filename */
  fileName = (char *)malloc( MAX_FILENAME_LEN * sizeof( char ));

  /* convert algorithm to string */
  algorithmName = convertAlgorithmToString( algorithm );
  dataTypeName = convertDataTypeToString( dataType );

  /* build path/filename for corresponding parameters */
  sprintf( fileName, "%s%s_%s.stats", STAT_FILE_HOME,
           algorithmName, dataTypeName );

  /* remember to free memory allocated by convertAlgo.. */
  free( algorithmName );
  free( dataTypeName );
  
  return( fileName );
}



/*
 * convertDataTypeToString
 */
char *
convertDataTypeToString( int dataType )
{
  char *dataType_string;

  dataType_string = (char *)malloc( MAX_FILENAME_LEN * sizeof( char ));
  
  if( dataType == REAL_DATA )
    strcpy( dataType_string, STAT_FILE_DATA_TYPE_NAME_REAL );
  else
    strcpy( dataType_string, STAT_FILE_DATA_TYPE_NAME_COMPLEX );
/*   printf(" data type = %d file ext = %s\n",dataType,dataType_string); */

  return( dataType_string );
}



/*
 * convertAlgorithmToString
 *
 * This function takes the integer reprsentation and
 * returns the string reprsentation used as the extension
 * on the simulation files. This routine allocates memory
 * for the filename to be returned.  The calling routine
 * should free this memory after use.
 */
char *
convertAlgorithmToString( int algorithm )
{
  char *algorithmName;

  algorithmName = (char *)malloc( MAX_FILENAME_LEN * sizeof( char ));
  
  switch( algorithm ) {
  case DFT:
    strcpy( algorithmName, DFT_FILENAME );
    break;
  case RADIX2:
    strcpy( algorithmName, RADIX2_FILENAME );
    break;
  case RADIX4:
    strcpy( algorithmName, RADIX4_FILENAME );
    break;
  case SPLITRADIX:
    strcpy( algorithmName, SPLITRADIX_FILENAME );
    break;
  case QFT:
    strcpy( algorithmName, QFT_FILENAME );
    break;
  case DITF:
    strcpy( algorithmName, DITF_FILENAME );
    break;
  case FHT:
    strcpy( algorithmName, FHT_FILENAME );
    break;
  default:
    printf("couldn't determine simulation filename\n");
    strcpy( algorithmName, FHT_FILENAME );
    break;
  }

  return( algorithmName );
}
 


/*
 * setButtonState
 *
 * This routine set the sensitivity of the run, step,
 * stop, and reset buttons depending on whether or not
 * the GUI is currently running an analysis simulation.
 * When the analysis is running the run, step, and reset
 * buttons are insensitive to input and the stop button is
 * sensitive. Vice Versa for when the GUI is not running
 * a simulation.
 */
void
setButtonsState( int running )
{
  XtSetSensitive( pushButton_job_run, !running );
  XtSetSensitive( pushButton_job_stepThru, !running );
  XtSetSensitive( pushButton_job_stop, running );
  XtSetSensitive( pushButton_window_reset, !running );
  XtSetSensitive( pushButton_dataSize_inc, !running );
  XtSetSensitive( pushButton_dataSize_dec, !running );
  XtSetSensitive( pushButton_numberOfProcessesUsed_inc, !running );
  XtSetSensitive( pushButton_numberOfProcessesUsed_inc, !running );
  XtSetSensitive( pulldown_algorithm, !running );
}


/*
 *
 * readAlgorithmSetting
 *
 * This function reads the alogorithm chosen, the data size,
 * and the number of processes from the GUI.
 *
 */

int
readAlgorithmSetting( int *algorithm, int *dataSize, int *local_numberOfProcesses, int *algorithmForm, int *dataType )
{
  Widget algorithm_buttonChosen;


  /* Read the selected algorithm from pulldown menu */

  XtVaGetValues( pulldown_algorithm,
                 XmNmenuHistory,&algorithm_buttonChosen,
                 NULL);


  /* determine algorithm chosen */

  if( algorithm_buttonChosen == pushButton_algorithm_dft )
    *algorithm = DFT;
  if( algorithm_buttonChosen == pushButton_algorithm_radix2 )
    *algorithm = RADIX2;
  if( algorithm_buttonChosen == pushButton_algorithm_radix4 )
    *algorithm = RADIX4;
  if( algorithm_buttonChosen == pushButton_algorithm_splitradix )
    *algorithm = SPLITRADIX;
  if( algorithm_buttonChosen == pushButton_algorithm_qft )
    *algorithm = QFT;
  if( algorithm_buttonChosen == pushButton_algorithm_ditf )
    *algorithm = DITF;
  if( algorithm_buttonChosen == pushButton_algorithm_fht )
    *algorithm = FHT;


  /* read dataSize and numberOfProcesses */

  *dataSize = getIntFromTextField( textField_dataSize );
  *local_numberOfProcesses = getIntFromTextField( textField_numberOfProcessesUsed );

  /* read and set data type */
  
  if( XmToggleButtonGetState( toggleButton_dataType_real ))
    *dataType = REAL_DATA;
  else
    *dataType = COMPLEX_DATA;

  /* read and set algorithm form */
  if( *local_numberOfProcesses > 1 )
    *algorithmForm = SEQUENTIAL_FORM;
  else
    *algorithmForm = PARALLEL_FORM;
  
#ifdef DEBUGd
  printf("algorithm = %d   data size = %d  # of processes = %d dataType = %d  form = %d\n",
         *algorithm, *dataSize, *local_numberOfProcesses, *dataType, *algorithmForm);
#endif

  return( NO_ERROR );
}


/*
 * isAnalysisRunning
 *
 * This routine allows for another routine to determine the an
 * analysis is currently being run.  If an analysis is running,
 * the routine return TRUE. Otherwise, it returns false
 * of the 
 */
int
isAnalysisRunning( )
{
  return( !XtIsSensitive( pushButton_job_run ));
}

  
/*
 * readStatsFromFile
 *
 */

int
readStatsFromFile( char *inFileName, STATS_TYPE *stats, int dataSize )
{
  FILE *fin;
  int lineNumber;
  int i;
  int order;
  int floatMults;
  int floatAdds;
  int intMults;
  int intAdds;
  int numberOfShifts;
  int memoryUsed;
  float timeTaken;

  /* open input file for reading */
  if(( fin = fopen( inFileName, "r" )) == NULL ) {
      printf( "ERROR opening file : %s", inFileName );
      fclose( fin );
      return( ERROR );
  }

  /* determine what line number we need */
  lineNumber = whatPowerOfTwo( dataSize );

  /* read until we get to the correct line number */
  for( i = 0; i < lineNumber; i++ )
    fscanf( fin, "%d %d %d %d %d %d %d %f", &order, &floatMults, &floatAdds, &intMults,
            &intAdds, &numberOfShifts, &memoryUsed, &timeTaken );
/*   printf( "%d %d %d %d %d %d %d %f\n", order, floatMults, floatAdds, intMults,
          intAdds, numberOfShifts, memoryUsed, timeTaken );*/

  /* fill structure with correct values */
  stats->float_mults = floatMults;
  stats->float_adds = floatAdds;
  stats->int_mults = intMults;
  stats->int_adds = intAdds;
  stats->memory_used = memoryUsed;
  stats->number_of_shifts = numberOfShifts;

  /* close file */
  fclose( fin );
}



/*
 * findPowerOfTwo
 *
 * This routine finds a power of two that is less than or equal to
 * the argument passed.
 */
int
findPowerOfTwo( int a_number )
{
  int powerOfTwo = 1;
  /* 
   *calculate the power of two that is <= current value
   */
  while( powerOfTwo <= a_number )
    powerOfTwo *= 2;
  powerOfTwo /= 2;

  return( powerOfTwo );
}


/*
 * whatPowerOfTwo
 *
 * This routine takes log2( a_number )
 */
int
whatPowerOfTwo( int a_number )
{
  int powerOfTwo = 1;
  int N = 0;
  /* 
   *calculate the power of two that is <= current value
   */
  while( powerOfTwo <= a_number ) {
    N++;
    powerOfTwo *= 2;
  }
  powerOfTwo /= 2;
  N--;

  return( N );
}
