/*
 * This file contains the graphics routines that are used by
 * the GUI for .
 */

#include<Xm/Xm.h>
#include<math.h>
#include "dspDemo_defines.h"
#include "dspDemo_externs.h"
#include "dspDemo_globals.h"

/*
 * FUNCTION PROTOTYPES
 */
int   readLogFile( char *inFileName );
int   calculatePercentTime( COMM_EVENT_TYPE *head_event_ptr );
void  processEvent( COMM_EVENT_TYPE *event_ptr );
void  initGraph( );
void  showBroadcast( MY2D source );
void  showMessageSend( MY2D source, MY2D destination );
void  showMessageReceive( MY2D destination );
int   setForegroundColor( String colorString );
void  setProcessState( int state, MY2D process );
void  drawProcess( MY2D process );
void  showProcessBusy( MY2D process );
void  showProcessDead( MY2D process );
void  drawProcessNumber( MY2D process, float angle, int nodeNumber );
void  drawNumbers( );
int   setEventTimer( int scale_factor );
int stopEventTimer( );


/*
 * setEventTimer
 *
 * This routine starts the interrupt alarm timer for the
 * next event in the event list.
 * NOTE: The time taken is scaled by the square of the scale
 *       factor provided
 */
int
setEventTimer( int scale_factor )
{
  unsigned int timeout;
  int temp_time;

  /* calculate time to wait to next event */
/*  printf("current time = %f prev time = %f\n", current_event->time,previous_event_time );*/
  temp_time = (int)((current_event->time - previous_event_time)* scale_factor);
/*  timeout = (unsigned int)(temp_time * scale_factor) ;*/
  timeout = (unsigned int)temp_time;

  /* enforce minimum time delay */
  if( timeout == 0 ) {
    timeout = (unsigned int)10;
/*    printf("enforcing min. timeout \n"); */
  }
  
  /* printf("next event in %u\n",timeout); */

  /* set the timer */
  ualarm( timeout, timeout );

  /* save this time for next event */
  previous_event_time = current_event->time;

  return( NO_ERROR );
}

/*
 * stopEventTimer
 *
 */
int
stopEventTimer( )
{
  ualarm(0,0);
  previous_event_time = 0.0;
/*   printf("reseting timer \n"); */
}

  
 




/*
 * readLogFile
 *
 */
int
readLogFile( char *inFileName )
{
/*   char inFileName[32] = "test.in"; */
  FILE *fin;
  COMM_EVENT_TYPE *event_ptr, *previous_event;
  
  /* try to open input file */
  
  if(( fin = fopen( inFileName, "r" )) == NULL ) {
    printf("ERROR opening logfile : %s", inFileName );
    fclose( fin );
    return( ERROR );
  }

  /* read the input file in */
  event_list = event_ptr = ( COMM_EVENT_TYPE *)malloc( sizeof( COMM_EVENT_TYPE ));
  while( fscanf( fin, "%d %f %d %d", &event_ptr->eventType, &event_ptr->time,
                 &event_ptr->source, &event_ptr->destination) != EOF )
  {
    event_ptr->next = ( COMM_EVENT_TYPE *)malloc( sizeof( COMM_EVENT_TYPE ));
    previous_event = event_ptr;
    event_ptr = event_ptr->next;
  }
  free( event_ptr );
  previous_event->next = NULL;
  previous_event = NULL;

  /* initialize current event */
  current_event = event_list;

  /* close file */

  fclose( fin );
#define DEBUG_READ_LOGFILEss
#ifdef DEBUG_READ_LOGFILE
  /* spit data back out */
  while( current_event != NULL )  {
    printf( " %d %f %d %d\n", current_event->eventType,
            current_event->time,current_event->source,
            current_event->destination);
    current_event = current_event->next;
  }
  current_event = event_list;
#endif  
}

/*
 * calculatePercentTime
 */

int
calculatePercentTime( COMM_EVENT_TYPE *event_ptr )
{
  float totalTime;

  if( event_ptr == NULL )
    return( FALSE );
  
  /* first get stop time */
  while( event_ptr->next != NULL )
    event_ptr = event_ptr->next;

  totalTime = event_ptr->time;
  
  /* calculate percent time for each event */
  event_ptr = event_list;
  while( event_ptr != NULL ) {
    event_ptr->percentTime = (int)(100*event_ptr->time/ totalTime);
#define PRINT_PERCENT_TIMEs
#ifdef PRINT_PERCENT_TIME    
    printf(" %time = %i\n",event_ptr->percentTime);
#endif    
    event_ptr = event_ptr->next;
  }
}

/*
 * processEvent
 *
 *
 */
void
processEvent( COMM_EVENT_TYPE *event_ptr )
{
/*   printf("processing event \n"); */
#define DEBUG_PROCESS_EVENTs
  switch( event_ptr->eventType ) {
  case SEND_EVENT:
#ifdef DEBUG_PROCESS_EVENT
    printf("%d SEND_EVENT %d\n",event_ptr->source,event_ptr->destination );
#endif    
    if( event_ptr->destination == ALL_NODES)  {
#ifdef DEBUG_PROCESS_EVENT
      printf("bRoAdCaSt %d\n",event_ptr->source);
#endif      
      showBroadcast( process[event_ptr->source] );
    }
    else{
#ifdef DEBUG_PROCESS_EVENT
      printf("simple send %d %d\n",event_ptr->source, event_ptr->destination);
#endif      
      showMessageSend( process[event_ptr->source],process[event_ptr->destination] );
    }
    break;
  case RECV_EVENT:
#ifdef DEBUG_PROCESS_EVENT
    printf("%d RECV_EVENT %d\n",event_ptr->source,event_ptr->destination );
#endif    
    showMessageReceive( process[event_ptr->source] );
    break;
  case WORKING_EVENT:
#ifdef DEBUG_PROCESS_EVENT
    printf("%d WORKING %d\n",event_ptr->source, event_ptr->destination );
#endif    
    showProcessBusy( process[event_ptr->source] );
    break;
  case START_EVENT:
#ifdef DEBUG_PROCESS_EVENT
    printf("%d START %d\n",event_ptr->source, event_ptr->destination );
#endif    
    showProcessBusy( process[event_ptr->source] );
    break;
  case STOP_EVENT:
#ifdef DEBUG_PROCESS_EVENT
    printf("%d STOP %d\n",event_ptr->source,event_ptr->destination );
#endif    
    showProcessDead( process[event_ptr->source] );
    break;
  default:
    printf("EVENT_ERROR: TYPE = %d\n",event_ptr->eventType );
  }
/*   printf("end process event\n"); */
}
/*
 * showMessageSend
 *
 *  This routine draws a line to indicate communcation
 *   between nodes.
 *
 *  INPUT : source : struct containing the coordinates of the
 *                   sending process
 */

void
showMessageSend( MY2D source, MY2D destination ){
  setProcessState( SEND_EVENT, source );
  XDrawLine( display, win, gc,
             source.x + 10, source.y + 10,
             destination.x + 10, destination.y + 10 );
}


/*
 * showMessageReceive
 *
 *  This routine shows a process receiving a message
 *
 *  INPUT: destination : struct containing the coordinate of
 *                       of the receiving process
 */

void
showMessageReceive( MY2D process ){
  setProcessState( RECV_EVENT, process );
}

/*
 * showProcessBusy
 * 
 */
void
showProcessBusy( MY2D process ) {
  setProcessState( WORKING_EVENT, process );
}

  
/*
 * showProcessDead
 * 
 */
void
showProcessDead( MY2D process ) {
  setProcessState( STOP_EVENT, process );
}  

/*
 * drawProcessNumbers
 *
 */

void
drawProcessNumerals( MY2D process, float angle, int nodeNumber ) 
{
  char nodeNumber_string[4];
  int length;
  int npro_2 = numberOfProcesses / 2;
  
  if( nodeNumber > 9 )
    length = 2;
  else
    length = 1;
  sprintf( nodeNumber_string, "%i", nodeNumber );
  if( nodeNumber == 0 ) {
    XDrawString( display, win, gc, process.x + 25, process.y + 13,
                 nodeNumber_string, length );
    return;
  }
  if( nodeNumber == npro_2 ) {
    XDrawString( display, win, gc, process.x - 15, process.y + 13,
                 nodeNumber_string, length );
    return;
  }
  
    
  if(( nodeNumber < npro_2 )&&( nodeNumber != 0)) {
    XDrawString( display, win, gc, process.x + 5, process.y + 35,
                 nodeNumber_string, length );
  }
  if( nodeNumber > npro_2 ) {
    XDrawString( display, win, gc, process.x, process.y - 10,
                 nodeNumber_string, length );
    
  }
}



/*
 *
 *
 */
void
drawLegend( )
{
  MY2D temp_process;
  

  /* display Initial State description */
  temp_process.x = 10;  temp_process.y = 25;
  setProcessState( START_EVENT, temp_process );
  setForegroundColor( TEXT_COLOR );
  XDrawString( display, win, gc, temp_process.x, temp_process.y-5,
               INIT_STATE_DEF, strlen(INIT_STATE_DEF));
  

  /* display Send State description */
  temp_process.x = 10;  temp_process.y = 95;
  setProcessState( SEND_EVENT, temp_process );
  setForegroundColor( TEXT_COLOR );
  XDrawString( display, win, gc, temp_process.x, temp_process.y-5,
               SEND_STATE_DEF, strlen(SEND_STATE_DEF));


  /* display Recv State description */
  temp_process.x = 10;  temp_process.y = 165;
  setProcessState( RECV_EVENT, temp_process );
  setForegroundColor( TEXT_COLOR );
  XDrawString( display, win, gc, temp_process.x, temp_process.y-5,
               RECV_STATE_DEF, strlen(RECV_STATE_DEF));  


  /* display Busy State description */
  temp_process.x = 10; temp_process.y = 235;
  setProcessState( WORKING_EVENT, temp_process );
  setForegroundColor( TEXT_COLOR );
  XDrawString( display, win, gc, temp_process.x, temp_process.y-5,
               WORKING_STATE_DEF, strlen(WORKING_STATE_DEF));  


  /* display Stop State description */
  temp_process.x = 10; temp_process.y = 305;
  setProcessState( STOP_EVENT, temp_process );
  setForegroundColor( TEXT_COLOR );
  XDrawString( display, win, gc, temp_process.x, temp_process.y -5,
               STOP_STATE_DEF, strlen(STOP_STATE_DEF));  
}

  
/*
 * InitGraph 
 * 
 * This function initalizes the processors (circles). 
 *
 */ 

void
InitGraph( ){
  int i;
  int radius;
  float theta;

  radius = WINDOW_WIDTH / 4;

/* Initialize the coordinates of the points */
  setForegroundColor( STOP_COLOR );
  theta = 2 * M_PI /  numberOfProcesses;
  for( i = 0; i < numberOfProcesses; i++ ){
    process[i].x = centerOfArea.x + (int)(radius * cos( i * theta ));
    process[i].y = centerOfArea.y + (int)(radius * sin( i * theta ));
    drawProcessNumerals( process[i], i * theta , i);
  }

  setForegroundColor( INIT_COLOR );
  /* Draw filled circles for process */
  for( i = 0; i < numberOfProcesses; i++ )
    drawProcess( process[i] );

  drawLegend( );
}    
      

/*
 * setProcessState
 *
 * This routine sets the state of the process
 *  by altering to foreground color and re-drawing
 *  the process.
 * NOTE: The foreground color is ALTERED by this routine.
 *       The routine does NOT restore the foreground color.
 *
 * INPUT: state : the state to which the process is to be set
 *        process : the process whose state is to be changed
 */
void
setProcessState( int state, MY2D process )
{
  switch( state ) {
  case SEND_EVENT:
    setForegroundColor( SENDING_COLOR );
    break;
    
  case RECV_EVENT:
    setForegroundColor( RECEIVING_COLOR );
    break;

  case WORKING_EVENT:
    setForegroundColor(  BUSY_COLOR );
    break;

  case STOP_EVENT:
    setForegroundColor( STOP_COLOR );
    break;
    
  defalt:
    setForegroundColor( STOP_COLOR );
    break;
  }

  /* must redraw process to get change in color */
  drawProcess( process );
}


/*
 * showBroadcast
 * 
 * This function draws the lines from a process
 *  to all other processes to indicate the communication
 *  between that process and all other process that occur
 *  during a broadcast
 */
void
showBroadcast( MY2D sendingProcess )
{
  int i;
  setProcessState( SEND_EVENT, sendingProcess );
/*  printf("broadcast:sending - %d %d\n",sendingProcess.x, 
         sendingProcess.y); */
  for( i = 0; i < numberOfProcesses; i++ )
    showMessageSend( sendingProcess, process[i] );
}

/*
 * setForegroundColor
 *
 * This function sets the color of items that will be drawn.
 * That is, it only affects the color of the item drawn after
 * calling this routine.
 */
int
setForegroundColor( String colorString )
{
  Colormap colorMap = DefaultColormapOfScreen( screen );
  XColor xcolor, unused;

  if( !XAllocNamedColor( display, colorMap, colorString,
                         &xcolor, &unused )) {
    char buf[32];
    sprintf( buf, "Cant alloc %s", colorString);
    XtWarning( buf );
    return( FALSE );
  }
  fillPixel = xcolor.pixel;
  XSetForeground( display, gc, fillPixel );
  return( TRUE );
}

/*
 * drawProcess
 *
 *  This function draws a circle to represent a process.
 *  The global variables used by this function are
 *
 *     display : a pointer to the terminal's display
 *     win     : a pointer to the window of the display
 *     gc      : the Graphics Context to be used
 *     arc_width,
 *     arc_height: the are equal & specify the radious of the circle
 *     ANGLE_0,
 *     ANGLE_360 : #defines that describe a closed circle (not a "pacman")
 *
 *    INPUT : a structure cotaining the coordinates of the location
 *            for the process to be drawn
 */

void
drawProcess( MY2D process )
{
  XFillArc( display, win , gc,
            process.x, process.y,
            arc_width, arc_height,
            ANGLE_0, ANGLE_360 );
}
