/*
 * dspDemo_initializations.c
 *
 * This file contains the functions to initialize the GUI
 *
 */

#include<Xm/Xm.h>
#include<signal.h>
#include "dspDemo_defines.h"
#include "dspDemo_externs.h"
#include "dspDemo_globals.h"
#include "dspDemo_signals.c"

/*
 * FUNCTION PROTOTYPES
 */
void InitSignalHandlers( );
void InitDisplayPointers( );
void InitGlobalVars( );
void InitStats( );


/*
 * InitSignalHandlers
 *
 * This function will initialize the signal handlers used by
 *  the GUI.  SIGUSR1 is used to trigger an update of the progress
 *  bar of an executing Fourier Transform Routine.
 */

void InitSignalHandlers()
{
  struct sigaction nvec;
  printf("Install Signal Handlers\n");

  sigemptyset( &nvec.sa_mask );
  nvec.sa_flags = 0;

  nvec.sa_handler = signalHandler_SIGALRM;
  sigaction( SIGALRM, &nvec, NULL );
  
  nvec.sa_handler = signalHandler_SIGUSR;
  sigaction( SIGUSR1, &nvec, NULL );

  nvec.sa_handler = signalHandler_SIGCHLD;
  sigaction( SIGCHLD, &nvec, NULL );
 
  
}

/*
 *   InitDisplayPointers
 * 
 *  NOTE: display,win,gc,screen are global
 */

void InitDisplayPointers( )
{
    XmDrawingAreaCallbackStruct da_struct;

  printf("Initializing Display Pointers\n");
  display = XtDisplay( drawingArea );
  screen = XtScreen( drawingArea );
  win = XtWindow( drawingArea );
  
  XtVaSetValues( appshell,
                 XmNworkWindow, drawingArea,
                 NULL );

  gcv.foreground = BlackPixelOfScreen( screen );
  gc = XCreateGC( display,
                  RootWindowOfScreen( screen ),
                  GCForeground,
                  &gcv );
}

/*
 * InitGlobalVars
 */ 
  
void InitGlobalVars( )
{
  printf("Initializing Globals \n");
  width  = 30;
  height = 20;
  x_coord = 30;
  y_coord = 30;
  rows_used = 1;
  centerOfArea.x = 350;
  centerOfArea.y = 170;
  arc_width = arc_height = 20;
  fillPixel = 1;     /* default to black */
  event_list = NULL;
  current_event = NULL;
  numberOfProcesses = 2;
  previous_event_time = 0.0;
  updateStatistics( );
  setNewBaseline( );
}
